/**************************************************************************

	File : getdata.h
	Author : Ran Nathan
	Date : October 24, 1999

	Overview:
	This file contains the definations needed to use the functions
	used to get the input data to run WINDISPER.
	Default values suggested below are those estimated for Aleppo pine (Pinus
	halepensis Mill.) at Nir-Ezyon (NE) study site, as described in the paper.

**************************************************************************/

typedef enum {
	FALSE=0,
	TRUE=1
} BOOL;

typedef struct {
/*  The simulated landscape */
	BOOL source;
	/*  TRUE if the cell is a "source" (adult tree canopy), FALSE if "empty" */
	unsigned int this_period_seeds;
	/*  Counts the number of seeds deposited in a cell */
	unsigned int all_periods_seeds;
} site_t;

#define WIND_SECTORS                        8
/*  Number of wind sectors, according to the main eight compass directions */
#define WIND_ARRAY_PARAMETERS               5
/*  Number of parameters in the wind data array */
#define TREE_ARRAY_PARAMETERS               4
/*  Number of parameters in the tree data array */
#define CELL_WIDTH                          1.0
/*  The actual width of a single resolution unit (a cell) in meters */
#define DEFAULT_NUMBER_OF_PERIODS           1
/*  The number of periods to be simulated */
#define DEFAULT_PERIOD_INPUT_FILE           "period"
/*  The (string part of the) name of the period input file */
#define DEFAULT_PERIOD_OUTPUT_FILE          "output"
/*  The (string part of the) name of the period output file */

/* CONSTANTS RELATED TO DISPERSAL PARAMETERS */
#define DEFAULT_MEAN_TREE_HEIGHT                    9.09
#define DEFAULT_STD_TREE_HEIGHT                     1.93
/* Mean and std of tree height (m)
	 The values 9.09 and 1.93 were estimated from measurement in 16 trees in NE */
#define DEFAULT_MEAN_PORTION_HEIGHT				       0.61
#define DEFAULT_STD_PORTION_HEIGHT				       0.07
/*  The mean and std of the prorotional distribution of cones (assumed = seeds)
	over tree height
	The values 0.61 and 0.07 were estimated from measurement in 16 trees	in NE */
#define DEFAULT_MIN_PORTION_HEIGHT			    	    0.01
#define DEFAULT_MAX_PORTION_HEIGHT			    	    0.99
/*  Minimum and maximum of the above parameters */
#define DEFAULT_MEAN_TERMINAL_VELOCITY              0.81
#define DEFAULT_STD_TERMINAL_VELOCITY               0.10
/*  The mean and std of the terminal velocity of a seed during fall [m/s]
	The values 0.81 and 0.1 are from Nathan et al. 1996 */
#define DEFAULT_PORTION_WINGLESS                    0.0
/*  The proportion of wingless seeds in the population studied.
	A wingless seed is assumed to fall in a completely vertical line,
	i.e., D=0.0. This parameter was not measured for P. halepensis */
#define DEFAULT_THRESHOLD_WINDSPEED                 0.0
/*  The minimal horizontal windspeed required to release a seed from a cone [m/s]
	This parameter was not measured for P. halepensis */
#define DEFAULT_MEAN_SEED_RELEASE                   2.09
#define DEFAULT_STD_SEED_RELEASE                    1.12
/*  The mean and std of the rate of seed release form a spatial resolution unit
	per day during the simulated period [seeds/(m^2*day)]
	The values 2.09 and 1.12 are based on seed-trap data collected during six
	succesive dispersal seasons in NE */

/* CONSTANTS RELATED TO METEOROLOGICAL PARAMETERS */
#define DEFAULT_MEAN_VERTICAL_WINDSPEED             0.10
#define DEFAULT_STD_VERTICAL_WINDSPEED              0.35
/*  The mean and std of the vertical windspeed [m/s]
	The values 0.10 and 0.35 were estimated from field measurements taken in
	NE during fall 1995 (n=10966)   */
#define DEFAULT_WS_ROUGHNESS                        0.07
#define DEFAULT_SITE_ROUGHNESS                      0.21
#define DEFAULT_WS_DISPLACEMENT                     0.30
#define DEFAULT_SITE_DISPLACEMENT                   0.36
/*  The two roughness parameters - roughness length and displacement
	height (both in m) for the reference weather station (WS), and for the
	simulated site (SITE).
	The values 0.07 and 0.30 for WS were taken from Wieringa 1993, the values
	0.21 and 0.36 were estimated from field measurements taken in Nir-Ezyon
	site in fall 1995 (n=10966), using the method of Robinson (1962), slightly
	modified by Haenel (1993). */
#define DEFAULT_MEASUREMENT_HEIGHT                 10
/*  The height where wind data is measured in the reference station */
#define DEFAULT_MEAN_LN_HORIZONTAL_WINDSPEED           0.93
#define DEFAULT_STD_LN_HORIZONTAL_WINDSPEED            0.66
#define DEFAULT_MAX_LN_HORIZONTAL_WINDSPEED            2.50
#define DEFAULT_MIN_LN_HORIZONTAL_WINDSPEED           -2.30
/*  The mean, std, max and min of the ln-transformed horizontal windspeed [m/s]
	The values 0.93, 0.66, 2.50 and -2.30 were estimated from wind measurements
	taken in	IMS weather station at Ein-Karmel, Israel, during 6 subsequent
	dispersal seasons (fall 1993 - spring 1996). */

int Get_All_Periods_Data(void);

/*************************************************************************

	Reads data for general parameters that are NOT changed during the
	periodic simulations.
	Returns the number of periods to simulated.

	File name is always "windisp.dat".

	The structure of the file (example):

	Tree distribution data is grid?	=	FALSE
	Number of periods	=	2
	Name of period input file	=	period
	Name of period output file	=	output
	Generate periodic output files?	=	TRUE
	Wind is random?	=	TRUE
	Mean tree height	=	10.0
	Std tree height	=	1.0
	Mean portion height	=	0.6
	Std portion height	=	0.05
	Max portion height	=	0.99
	Min portion height	=	0.01
	Mean terminal velocity	=	1.0
	Std terminal velocity	=	0.12
	Portion wingless	=	0.0
	Threshold windspeed	=	0.0
	Mean vertical windspeed	=	0.1
	Std vertical windspeed	=	0.05
	Study site roughness	=	0.5
	Study site displacement	=	1.0
	Weather station roughness	=	0.2
	Weather station displacement	=	0.4
	Wind measurement height	=	10.0

	Important notes:
	1. "Tree distribution data is grid?" is critical, since the program cannot
		run without the information on tree distribution. Write TRUE if this
		information should be read from an ARC/Info generated grid file, and FALSE
		if from X-Y-radius data file.
	2.	"Generate periodic output files?" - the answer is typically FALSE,
		especially if many periods are simulated, since the output files (in
		the ARC/Info format) are typically very large.
	3. "Wind is random" - if the answer is TRUE, the program selects a random
		wind direction and a random horizontal windspeed from the periodic
		averages, if available (otherwise it is selected based on the default
		values, which are the based on much more extensive data) . If the answer
		is FALSE, a file with directional wind data file *.wnd - see the function
		Get_This_Period_Data() below) is necessary.
	4. "Wind measurement height" must be > "Weather station displacement".

**************************************************************************/


int Get_This_Period_Data(char*, float*, float*, float*, float*, float*, float*);

/*************************************************************************

	Reads data for parameters that may change during the periodic simulations.
	Returns the number of days in the simulated period.

	For each simulated period, opens two files *.dat and, optionally (if
	Wind_Is_Random = FALSE; see function Get_This_Period_Data() above), *.wnd,
	where * is the	name specified for this period (e.g., "period1") in
	Get_All_Periods_Data().


	The structure of the input file *.dat (example):

		Days in this period	=	7
		Mean seed release	=	1
		Std seed release	=	0.05
		Mean ln horizontal windspeed	=	1.2029
		Std ln horizontal windspeed	=	0.6123
		Max ln horizontal windspeed	=	2.2925
		Min ln horizontal windspeed	=	-1.204

	Important notes:
	1. "Mean seed release" is the estimated number of seeds released per day,
		during the simulated period, from a quadratic parcel whose base is equal
		to the spatial resolution unit (Cell_Width^2, typically 1 m^2), and its
		height is the tree height.


	The structure of the input file *.wnd:

		Contains 8 rows for each of the 45 degree wind direction	sectors
		(first - 337.5 to 22.5, second - 22.5 to 77.5, etc.). Each row should
		have 5 columns for:
			1) COMULATIVE frequency of wind measured (blowing from) that direction.
				(NOTE: the last value must be 1.0000!)
			2) Mean of ln-transformed horizontal windspeed (since the distribution
					of horizontal windspeed is typically log-normal).
			3) Std of ln-transformed horizontal windspeed.
			4) Maximum of ln-transformed horizontal windspeed.
			5) Minimum of ln-transformed horizontal windspeed.

		Example:
		0.0494	1.1005	0.3929	1.7047	0
		0.0972	0.5546	0.4089	1.3083	-0.2231
		0.2531	1.367		0.8407	2.2925	-0.9163
		0.5262	0.948		0.4414	2.0541	-0.3567
		0.6018	1.1218	0.5717	2.1163	-0.6931
		0.7453	1.5686	0.548		2.2824	-0.9163
		0.8672	1.4158	0.4734	1.9459	-0.9163
		1.0000	1.2615	0.5175	1.8245	-1.204

**************************************************************************/


void Get_Trees_Grid(void);

/*************************************************************************

	Reads the location of adult (seed-releasing) trees within the simulated site
	from an ARC/Info-generated ASCII file,	e.g., one processed from a recent
	aerial photo of the site.

	After 6 rows of general information (see below), each cell within the
	ncols x nrows lattice gets either 0 if "empty" and 1 if "source"
	(seed-releasing).

	File name is always "trees.grd".

	File structure (example):

	ncols	          20
	nrows	          10
	xllcorner       147300
	yllcorner       232500
	cellsize        1
	NODATA_value    -9999
	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0
	0	0	0	1	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0
	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0
	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0
	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0
	0	0	0	0	0	0	0	0	0	0	1	1	1	1	0	0	0	0	0	0
	0	0	0	0	0	0	0	0	0	0	1	1	1	1	0	0	0	0	0	0
	0	0	0	0	0	0	0	0	0	0	1	1	1	1	0	0	0	0	0	0
	0	0	0	0	0	0	0	0	0	0	1	1	1	1	0	0	0	0	0	0
	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0

**************************************************************************/


void Get_Trees_XY(void);

/*************************************************************************

	Reads the location adult (seed-releasing)	trees in the simulated site.

	After 4 rows of general information (see below), each row refers for each
	tree, giving the x-y coordinates of its center, and then the radius,
	assuming perfectly circled canopy.

	File name is always "treesxy.dat".

	File structure (example):

	ncols	          50
	nrows	          50
	cellsize        1
	num_trees		 5
	Tree	x	y	radius
	1		20	10		5
	2		35	10		6
	3		15	28		7
	4		30	27		8
	5		30	36		5

**************************************************************************/

void Reset_Landscape(void);

/*************************************************************************
	Resets the simulated landscape.

**************************************************************************/

